
import java.util.Set;
import java.util.Iterator;
import java.util.Random;
import java.io.PrintStream;


public class TreeSet
{
    // Wurzel des Binaerbaums fuer eine Menge.
    Entry root;
    
    public TreeSet() {
        root = null;        
    }
    
    
    public boolean add(Object o) {
        if (root == null) {
            root = new Entry(o, null);
            
            return true;
        }
        
        return addToTree(o, root);
    }
    
    
    private boolean addToTree(Object o, Entry entry) {
        int result = ((Comparable) entry.value).compareTo(o);
        
        if (result == 0) {
            return false;
        }
        
        // Abstieg im Baum um neues Element an richtiger
        // Position einzutragen.
        if (result > 0) {
            if (entry.left == null) {
                entry.left = new Entry(o, entry);
                return true;
            }
            
            return addToTree(o, entry.left);
        }
        
        if (entry.right == null) {
            entry.right = new Entry(o, entry);
            return true;
        }
        
        return addToTree(o, entry.right);
    }
    
    
    public boolean contains(Object o) {
        return treeContains(o, root);        
    }
    
    
    // Element suchen durch Abstieg im Baum
    private boolean treeContains(Object o, Entry entry)
    {
        if (entry == null) return false;
        
        int result = ((Comparable) entry.value).compareTo(o);
        
        if (result == 0) return true;
        if (result > 0) return treeContains(o, entry.left);
        return treeContains(o, entry.right);
    }
    
    
    public boolean remove(Object o) {
        return removeFromTree(o, root);
    }
    
    
    private boolean removeFromTree(Object o, Entry entry)
    {
        // Wenn der Eintrag nicht vorhande ist,
        // kann er nicht entfernt werden.
        if (entry == null) return false;
        
        int result = ((Comparable) entry.value).compareTo(o);
        
        // Abstieg zum zu loeschenden Eintrag:
        if (result > 0) return removeFromTree(o, entry.left);
        if (result < 0) return removeFromTree(o, entry.right);
        
        // Zu loeschender Eintrag gefunden.
        // Falls kein linker Nachfolger vorhanden...
        if (entry.left == null) {
            // ... zu loeschenden Eintrag durch rechten
            // Nachfolger ersetzen
            // Rechten Nachfolger anpassen.
            if (entry.right != null) {
                entry.right.parent = entry.parent;
            }
            // Vater anpassen.
            adjustParentEntry(entry, entry.right);            
        }
        else {
            // Linker Eintrag ist vorhanden.
            Entry maxEntry = entry.left;
            // Falls vorhanden, kleinsten Eintrag im rechten
            // Unterbaum (des linken Eintrags) suchen...
            if (maxEntry.right != null) {
                while (maxEntry.right != null) {
                    maxEntry = maxEntry.right;
                }
                // ... und zu loeschenden Eintrag mit diesem
                //     ersetzen.
                maxEntry.parent.right = null;
                entry.value = maxEntry.value;
            }
            else {
                // Andernfalls: linker Eintrag ersetzt zu
                // loeschenden Eintrag.
                maxEntry.parent = entry.parent;
                maxEntry.right = entry.right;
                if (entry.right != null) {
                    entry.right.parent = maxEntry;
                }
                adjustParentEntry(entry, maxEntry);
            }
        }
        
        // Ein Eintrag wurde geloescht.
        return true;
    }
    
    
    // Vatereintrag von zu loeschendem Eintrag anpassen.
    private void adjustParentEntry(Entry entry, Entry replacingEntry)
    {
        if (entry == root) {
            root = replacingEntry;
        }
        else if (entry.parent.left == entry) {
            entry.parent.left = replacingEntry;
        }
        else {
            entry.parent.right = replacingEntry;
        }
    }
    
    
    public Iterator iterator()
    {
        return new TreeSetIterator(this);
    }
    
    
    // Postorderdurchlauf ueber Binaerbaum liefert
    // Mengenelemente in sortierter Reihenfolge.
    public static final class TreeSetIterator implements Iterator
    {
        // Zu Traversierender Baum
        private TreeSet treeSet;
        // Aktuelle Position fuer Baumdurchlauf
        private Entry position;
        
        public TreeSetIterator(TreeSet treeSet) {
            this.treeSet = treeSet;
            // Erste Position ist das am weitesten
            // links liegende Blatt.
            position = treeSet.root;
            if (position != null) { 
                while (position.left != null) {
                    position = position.left;
                }
            }
        }
        
        public boolean hasNext() {
            return position != null;
        }
        
        public Object next() {
            // Merke zu liefernden Wert von aktueller
            // Position.
            Object result = position.value;
            
            // Berechne naechste Position:
            // Abstieg nach rechts nicht moeglich?
            if (position.right == null) {
                // Dann Aufstieg solange man von "rechts unten kommt"
                // (also auf dem Rueckweg der Traversierung ist...)
                // Geht nur, wenn man noch nicht bei der Wurzel ist.
                if (position == treeSet.root) {
                    position = null;
                }
                else {
                    Entry previousPosition = position;
                    position = position.parent;
                    while (position.right == previousPosition &&
                           position != treeSet.root) {
                        previousPosition = position;
                        position = position.parent;
                    }
                    if (position.right == previousPosition && 
                        position == treeSet.root) {
                        position = null;
                    }
                }
            }
            else {
                // Abstieg nach rechst und dort zum kleinsten
                // Element (am weitesten links im Baum.
                position = position.right;
                while (position.left != null) {
                    position = position.left;
                }
            }
            
            return result;
        }
        
        public void remove() 
        { 
            // Not yet implemented...
            throw new RuntimeException("Not yet implemented.");
        }
    }

    
    public static void main(String[] args) {
        Random random = new Random();
        TreeSet testSet = new TreeSet();
        
        for (int i = 0; i < 100; i++) {
            int n = random.nextInt() % 15;
            
            System.out.println("Adding " + n);
            testSet.add(new Integer(n));
            
            System.out.print("[");
            Iterator iterator = testSet.iterator();
            while (iterator.hasNext()) {
                Integer res = (Integer) iterator.next();
                
                System.out.print(res);
                System.out.print(" ");
            }
            System.out.println("]");

            n = random.nextInt() % 15;
            if (testSet.contains(new Integer(n))) {
                System.out.println("Deleting " + n);
                testSet.remove(new Integer(n));
                
                System.out.print("[");
                iterator = testSet.iterator();
                while (iterator.hasNext()) {
                    Integer res = (Integer) iterator.next();
                    
                    System.out.print(res);
                    System.out.print(" ");
                }
                System.out.println("]");
            }
        }
    }
}


class Entry {
    Entry left;
    Entry right;
    Entry parent;
    
    Object value;
    
    public Entry(Object o, Entry parent) 
    {
        left = right = null;
        value = o;
        this.parent = parent;
    }
}
